@@ -1,7 +1,7 @@
 /* efi.h - declare EFI types and functions */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2006,2007,2008  Free Software Foundation, Inc.
+ *  Copyright (C) 2006,2007,2008,2009  Free Software Foundation, Inc.
  *
  *  GRUB is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #define GRUB_EFI_API_HEADER	1
 
 #include <grub/types.h>
+#include <grub/symbol.h>
 
 /* For consistency and safety, we name the EFI-defined types differently.
    All names are transformed into lower case, _t appended, and
@@ -83,6 +84,21 @@
     { 0x8e, 0x39, 0x00, 0xa0, 0xc9, 0x69, 0x72, 0x3b } \
   }
 
+#define GRUB_EFI_SERIAL_IO_GUID \
+  { 0xbb25cf6f, 0xf1d4, 0x11d2, \
+    { 0x9a, 0x0c, 0x00, 0x90, 0x27, 0x3f, 0xc1, 0xfd } \
+  }
+
+#define GRUB_EFI_SIMPLE_NETWORK_GUID	\
+  { 0xa19832b9, 0xac25, 0x11d3, \
+    { 0x9a, 0x2d, 0x00, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
+  }
+
+#define GRUB_EFI_PXE_GUID	\
+  { 0x03c4e603, 0xac28, 0x11d3, \
+    { 0x9a, 0x2d, 0x00, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
+  }
+
 #define GRUB_EFI_DEVICE_PATH_GUID	\
   { 0x09576e91, 0x6d3f, 0x11d2, \
     { 0x8e, 0x39, 0x00, 0xa0, 0xc9, 0x69, 0x72, 0x3b } \
@@ -108,6 +124,127 @@
     { 0x9a, 0x16, 0x0, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
   }
 
+#define GRUB_EFI_SAL_TABLE_GUID \
+  { 0xeb9d2d32, 0x2d88, 0x11d3, \
+      { 0x9a, 0x16, 0x0, 0x90, 0x27, 0x3f, 0xc1, 0x4d } \
+  }
+
+#define GRUB_EFI_HCDP_TABLE_GUID \
+  { 0xf951938d, 0x620b, 0x42ef, \
+      { 0x82, 0x79, 0xa8, 0x4b, 0x79, 0x61, 0x78, 0x98 } \
+  }
+
+struct grub_efi_sal_system_table
+{
+  grub_uint32_t signature;
+  grub_uint32_t total_table_len;
+  grub_uint16_t sal_rev;
+  grub_uint16_t entry_count;
+  grub_uint8_t checksum;
+  grub_uint8_t reserved1[7];
+  grub_uint16_t sal_a_version;
+  grub_uint16_t sal_b_version;
+  grub_uint8_t oem_id[32];
+  grub_uint8_t product_id[32];
+  grub_uint8_t reserved2[8];
+  grub_uint8_t entries[0];
+};
+
+enum
+  {
+    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_ENTRYPOINT_DESCRIPTOR = 0,
+    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_MEMORY_DESCRIPTOR = 1,
+    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_PLATFORM_FEATURES = 2,
+    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_TRANSLATION_REGISTER_DESCRIPTOR = 3,
+    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_PURGE_TRANSLATION_COHERENCE = 4,
+    GRUB_EFI_SAL_SYSTEM_TABLE_TYPE_AP_WAKEUP = 5
+  };
+
+struct grub_efi_sal_system_table_entrypoint_descriptor
+{
+  grub_uint8_t type;
+  grub_uint8_t pad[7];
+  grub_uint64_t pal_proc_addr;
+  grub_uint64_t sal_proc_addr;
+  grub_uint64_t global_data_ptr;
+  grub_uint64_t reserved[2];
+};
+
+struct grub_efi_sal_system_table_memory_descriptor
+{
+  grub_uint8_t type;
+  grub_uint8_t sal_used;
+  grub_uint8_t attr;
+  grub_uint8_t ar;
+  grub_uint8_t attr_mask;
+  grub_uint8_t mem_type;
+  grub_uint8_t usage;
+  grub_uint8_t unknown;
+  grub_uint64_t addr;
+  grub_uint64_t len;
+  grub_uint64_t unknown2;
+};
+
+struct grub_efi_sal_system_table_platform_features
+{
+  grub_uint8_t type;
+  grub_uint8_t flags;
+  grub_uint8_t reserved[14];
+};
+
+struct grub_efi_sal_system_table_translation_register_descriptor
+{
+  grub_uint8_t type;
+  grub_uint8_t register_type;
+  grub_uint8_t register_number;
+  grub_uint8_t reserved[5];
+  grub_uint64_t addr;
+  grub_uint64_t page_size;
+  grub_uint64_t reserver;
+};
+
+struct grub_efi_sal_system_table_purge_translation_coherence
+{
+  grub_uint8_t type;
+  grub_uint8_t reserved[3];  
+  grub_uint32_t ndomains;
+  grub_uint64_t coherence;
+};
+
+struct grub_efi_sal_system_table_ap_wakeup
+{
+  grub_uint8_t type;
+  grub_uint8_t mechanism;
+  grub_uint8_t reserved[6];
+  grub_uint64_t vector;
+};
+
+enum
+  {
+    GRUB_EFI_SAL_SYSTEM_TABLE_PLATFORM_FEATURE_BUSLOCK = 1,
+    GRUB_EFI_SAL_SYSTEM_TABLE_PLATFORM_FEATURE_IRQREDIRECT = 2,
+    GRUB_EFI_SAL_SYSTEM_TABLE_PLATFORM_FEATURE_IPIREDIRECT = 4,
+    GRUB_EFI_SAL_SYSTEM_TABLE_PLATFORM_FEATURE_ITCDRIFT = 8,
+  };
+
+typedef enum grub_efi_parity_type
+  {
+    GRUB_EFI_SERIAL_DEFAULT_PARITY,
+    GRUB_EFI_SERIAL_NO_PARITY,
+    GRUB_EFI_SERIAL_EVEN_PARITY,
+    GRUB_EFI_SERIAL_ODD_PARITY
+  }
+grub_efi_parity_type_t;
+
+typedef enum grub_efi_stop_bits
+  {
+    GRUB_EFI_SERIAL_DEFAULT_STOP_BITS,
+    GRUB_EFI_SERIAL_1_STOP_BIT,
+    GRUB_EFI_SERIAL_1_5_STOP_BITS,
+    GRUB_EFI_SERIAL_2_STOP_BITS
+  }
+  grub_efi_stop_bits_t;
+
 /* Enumerations.  */
 enum grub_efi_timer_delay
   {
@@ -183,6 +320,8 @@ typedef grub_uint64_t grub_efi_uint64_t;
 typedef grub_uint8_t grub_efi_char8_t;
 typedef grub_uint16_t grub_efi_char16_t;
 
+#define PRIxGRUB_EFI_UINTN_T "lx"
+
 typedef grub_efi_intn_t grub_efi_status_t;
 
 #define GRUB_EFI_ERROR_CODE(value)	\
@@ -255,7 +394,7 @@ struct grub_efi_memory_descriptor
   grub_efi_virtual_address_t virtual_start;
   grub_efi_uint64_t num_pages;
   grub_efi_uint64_t attribute;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_memory_descriptor grub_efi_memory_descriptor_t;
 
 /* Device Path definitions.  */
@@ -300,7 +439,7 @@ struct grub_efi_pci_device_path
   grub_efi_device_path_t header;
   grub_efi_uint8_t function;
   grub_efi_uint8_t device;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_pci_device_path grub_efi_pci_device_path_t;
 
 #define GRUB_EFI_PCCARD_DEVICE_PATH_SUBTYPE		2
@@ -309,7 +448,7 @@ struct grub_efi_pccard_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_uint8_t function;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_pccard_device_path grub_efi_pccard_device_path_t;
 
 #define GRUB_EFI_MEMORY_MAPPED_DEVICE_PATH_SUBTYPE	3
@@ -317,10 +456,10 @@ typedef struct grub_efi_pccard_device_path grub_efi_pccard_device_path_t;
 struct grub_efi_memory_mapped_device_path
 {
   grub_efi_device_path_t header;
-  grub_efi_memory_type_t memory_type;
+  grub_efi_uint32_t memory_type;
   grub_efi_physical_address_t start_address;
   grub_efi_physical_address_t end_address;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_memory_mapped_device_path grub_efi_memory_mapped_device_path_t;
 
 #define GRUB_EFI_VENDOR_DEVICE_PATH_SUBTYPE		4
@@ -330,7 +469,7 @@ struct grub_efi_vendor_device_path
   grub_efi_device_path_t header;
   grub_efi_guid_t vendor_guid;
   grub_efi_uint8_t vendor_defined_data[0];
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_vendor_device_path grub_efi_vendor_device_path_t;
 
 #define GRUB_EFI_CONTROLLER_DEVICE_PATH_SUBTYPE		5
@@ -339,7 +478,7 @@ struct grub_efi_controller_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_uint32_t controller_number;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_controller_device_path grub_efi_controller_device_path_t;
 
 /* ACPI Device Path.  */
@@ -352,7 +491,7 @@ struct grub_efi_acpi_device_path
   grub_efi_device_path_t header;
   grub_efi_uint32_t hid;
   grub_efi_uint32_t uid;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_acpi_device_path grub_efi_acpi_device_path_t;
 
 #define GRUB_EFI_EXPANDED_ACPI_DEVICE_PATH_SUBTYPE	2
@@ -363,8 +502,8 @@ struct grub_efi_expanded_acpi_device_path
   grub_efi_uint32_t hid;
   grub_efi_uint32_t uid;
   grub_efi_uint32_t cid;
-  char hidstr[1];
-};
+  char hidstr[0];
+} __attribute__ ((packed));
 typedef struct grub_efi_expanded_acpi_device_path grub_efi_expanded_acpi_device_path_t;
 
 #define GRUB_EFI_EXPANDED_ACPI_HIDSTR(dp)	\
@@ -387,7 +526,7 @@ struct grub_efi_atapi_device_path
   grub_efi_uint8_t primary_secondary;
   grub_efi_uint8_t slave_master;
   grub_efi_uint16_t lun;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_atapi_device_path grub_efi_atapi_device_path_t;
 
 #define GRUB_EFI_SCSI_DEVICE_PATH_SUBTYPE		2
@@ -397,7 +536,7 @@ struct grub_efi_scsi_device_path
   grub_efi_device_path_t header;
   grub_efi_uint16_t pun;
   grub_efi_uint16_t lun;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_scsi_device_path grub_efi_scsi_device_path_t;
 
 #define GRUB_EFI_FIBRE_CHANNEL_DEVICE_PATH_SUBTYPE	3
@@ -408,7 +547,7 @@ struct grub_efi_fibre_channel_device_path
   grub_efi_uint32_t reserved;
   grub_efi_uint64_t wwn;
   grub_efi_uint64_t lun;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_fibre_channel_device_path grub_efi_fibre_channel_device_path_t;
 
 #define GRUB_EFI_1394_DEVICE_PATH_SUBTYPE		4
@@ -418,7 +557,7 @@ struct grub_efi_1394_device_path
   grub_efi_device_path_t header;
   grub_efi_uint32_t reserved;
   grub_efi_uint64_t guid;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_1394_device_path grub_efi_1394_device_path_t;
 
 #define GRUB_EFI_USB_DEVICE_PATH_SUBTYPE		5
@@ -428,7 +567,7 @@ struct grub_efi_usb_device_path
   grub_efi_device_path_t header;
   grub_efi_uint8_t parent_port_number;
   grub_efi_uint8_t interface;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_usb_device_path grub_efi_usb_device_path_t;
 
 #define GRUB_EFI_USB_CLASS_DEVICE_PATH_SUBTYPE		15
@@ -441,7 +580,7 @@ struct grub_efi_usb_class_device_path
   grub_efi_uint8_t device_class;
   grub_efi_uint8_t device_subclass;
   grub_efi_uint8_t device_protocol;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_usb_class_device_path grub_efi_usb_class_device_path_t;
 
 #define GRUB_EFI_I2O_DEVICE_PATH_SUBTYPE		6
@@ -450,7 +589,7 @@ struct grub_efi_i2o_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_uint32_t tid;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_i2o_device_path grub_efi_i2o_device_path_t;
 
 #define GRUB_EFI_MAC_ADDRESS_DEVICE_PATH_SUBTYPE	11
@@ -460,7 +599,7 @@ struct grub_efi_mac_address_device_path
   grub_efi_device_path_t header;
   grub_efi_mac_address_t mac_address;
   grub_efi_uint8_t if_type;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_mac_address_device_path grub_efi_mac_address_device_path_t;
 
 #define GRUB_EFI_IPV4_DEVICE_PATH_SUBTYPE		12
@@ -474,7 +613,7 @@ struct grub_efi_ipv4_device_path
   grub_efi_uint16_t remote_port;
   grub_efi_uint16_t protocol;
   grub_efi_uint8_t static_ip_address;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_ipv4_device_path grub_efi_ipv4_device_path_t;
 
 #define GRUB_EFI_IPV6_DEVICE_PATH_SUBTYPE		13
@@ -488,7 +627,7 @@ struct grub_efi_ipv6_device_path
   grub_efi_uint16_t remote_port;
   grub_efi_uint16_t protocol;
   grub_efi_uint8_t static_ip_address;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_ipv6_device_path grub_efi_ipv6_device_path_t;
 
 #define GRUB_EFI_INFINIBAND_DEVICE_PATH_SUBTYPE		9
@@ -501,7 +640,7 @@ struct grub_efi_infiniband_device_path
   grub_efi_uint64_t remote_id;
   grub_efi_uint64_t target_port_id;
   grub_efi_uint64_t device_id;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_infiniband_device_path grub_efi_infiniband_device_path_t;
 
 #define GRUB_EFI_UART_DEVICE_PATH_SUBTYPE		14
@@ -514,7 +653,7 @@ struct grub_efi_uart_device_path
   grub_efi_uint8_t data_bits;
   grub_efi_uint8_t parity;
   grub_efi_uint8_t stop_bits;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_uart_device_path grub_efi_uart_device_path_t;
 
 #define GRUB_EFI_VENDOR_MESSAGING_DEVICE_PATH_SUBTYPE	10
@@ -524,7 +663,7 @@ struct grub_efi_vendor_messaging_device_path
   grub_efi_device_path_t header;
   grub_efi_guid_t vendor_guid;
   grub_efi_uint8_t vendor_defined_data[0];
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_vendor_messaging_device_path grub_efi_vendor_messaging_device_path_t;
 
 /* Media Device Path.  */
@@ -541,7 +680,7 @@ struct grub_efi_hard_drive_device_path
   grub_efi_uint8_t partition_signature[8];
   grub_efi_uint8_t mbr_type;
   grub_efi_uint8_t signature_type;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_hard_drive_device_path grub_efi_hard_drive_device_path_t;
 
 #define GRUB_EFI_CDROM_DEVICE_PATH_SUBTYPE		2
@@ -552,7 +691,7 @@ struct grub_efi_cdrom_device_path
   grub_efi_uint32_t boot_entry;
   grub_efi_lba_t partition_start;
   grub_efi_lba_t partition_size;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_cdrom_device_path grub_efi_cdrom_device_path_t;
 
 #define GRUB_EFI_VENDOR_MEDIA_DEVICE_PATH_SUBTYPE	3
@@ -562,7 +701,7 @@ struct grub_efi_vendor_media_device_path
   grub_efi_device_path_t header;
   grub_efi_guid_t vendor_guid;
   grub_efi_uint8_t vendor_defined_data[0];
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_vendor_media_device_path grub_efi_vendor_media_device_path_t;
 
 #define GRUB_EFI_FILE_PATH_DEVICE_PATH_SUBTYPE		4
@@ -571,7 +710,7 @@ struct grub_efi_file_path_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_char16_t path_name[0];
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_file_path_device_path grub_efi_file_path_device_path_t;
 
 #define GRUB_EFI_PROTOCOL_DEVICE_PATH_SUBTYPE		5
@@ -580,7 +719,7 @@ struct grub_efi_protocol_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_guid_t guid;
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_protocol_device_path grub_efi_protocol_device_path_t;
 
 #define GRUB_EFI_PIWG_DEVICE_PATH_SUBTYPE		6
@@ -589,7 +728,7 @@ struct grub_efi_piwg_device_path
 {
   grub_efi_device_path_t header;
   grub_efi_guid_t guid __attribute__ ((packed));
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_piwg_device_path grub_efi_piwg_device_path_t;
 
 
@@ -604,7 +743,7 @@ struct grub_efi_bios_device_path
   grub_efi_uint16_t device_type;
   grub_efi_uint16_t status_flags;
   char description[0];
-};
+} __attribute__ ((packed));
 typedef struct grub_efi_bios_device_path grub_efi_bios_device_path_t;
 
 struct grub_efi_open_protocol_information_entry
@@ -909,9 +1048,13 @@ struct grub_efi_runtime_services
   grub_efi_status_t
   (*convert_pointer) (grub_efi_uintn_t debug_disposition, void **address);
 
+#define GRUB_EFI_GLOBAL_VARIABLE_GUID \
+  { 0x8BE4DF61, 0x93CA, 0x11d2, { 0xAA, 0x0D, 0x00, 0xE0, 0x98, 0x03, 0x2B,0x8C }}
+
+
   grub_efi_status_t
   (*get_variable) (grub_efi_char16_t *variable_name,
-		   grub_efi_guid_t *vendor_guid,
+		   const grub_efi_guid_t *vendor_guid,
 		   grub_efi_uint32_t *attributes,
 		   grub_efi_uintn_t *data_size,
 		   void *data);
@@ -949,6 +1092,27 @@ typedef struct grub_efi_configuration_table grub_efi_configuration_table_t;
 #define GRUB_EFIEMU_SYSTEM_TABLE_SIGNATURE 0x5453595320494249LL
 #define GRUB_EFIEMU_RUNTIME_SERVICES_SIGNATURE 0x56524553544e5552LL
 
+struct grub_efi_serial_io_interface
+{
+  grub_efi_uint32_t revision;
+  void (*reset) (void);
+  grub_efi_status_t (*set_attributes) (struct grub_efi_serial_io_interface *this,
+				       grub_efi_uint64_t speed,
+				       grub_efi_uint32_t fifo_depth,
+				       grub_efi_uint32_t timeout,
+				       grub_efi_parity_type_t parity,
+				       grub_uint8_t word_len,
+				       grub_efi_stop_bits_t stop_bits);
+  void (*set_control_bits) (void);
+  void (*get_control_bits) (void);
+  grub_efi_status_t (*write) (struct grub_efi_serial_io_interface *this,
+			      grub_efi_uintn_t *buf_size,
+			      void *buffer);
+  grub_efi_status_t (*read) (struct grub_efi_serial_io_interface *this,
+			     grub_efi_uintn_t *buf_size,
+			     void *buffer);
+};
+
 struct grub_efi_simple_input_interface
 {
   grub_efi_status_t
@@ -1007,6 +1171,36 @@ struct grub_efi_simple_text_output_interface
 };
 typedef struct grub_efi_simple_text_output_interface grub_efi_simple_text_output_interface_t;
 
+typedef grub_uint8_t grub_efi_pxe_packet_t[1472];
+
+typedef struct grub_efi_pxe_mode
+{
+  grub_uint8_t unused[52];
+  grub_efi_pxe_packet_t dhcp_discover;
+  grub_efi_pxe_packet_t dhcp_ack;
+  grub_efi_pxe_packet_t proxy_offer;
+  grub_efi_pxe_packet_t pxe_discover;
+  grub_efi_pxe_packet_t pxe_reply;
+} grub_efi_pxe_mode_t;
+
+typedef struct grub_efi_pxe
+{
+  grub_uint64_t rev;
+  void (*start) (void);
+  void (*stop) (void);
+  void (*dhcp) (void);
+  void (*discover) (void);
+  void (*mftp) (void);
+  void (*udpwrite) (void);
+  void (*udpread) (void);
+  void (*setipfilter) (void);
+  void (*arp) (void);
+  void (*setparams) (void);
+  void (*setstationip) (void);
+  void (*setpackets) (void);
+  struct grub_efi_pxe_mode *mode;
+} grub_efi_pxe_t;
+
 #define GRUB_EFI_BLACK		0x00
 #define GRUB_EFI_BLUE		0x01
 #define GRUB_EFI_GREEN		0x02
@@ -1108,6 +1302,76 @@ struct grub_efi_block_io_media
 };
 typedef struct grub_efi_block_io_media grub_efi_block_io_media_t;
 
+typedef grub_uint8_t grub_efi_mac_t[32];
+
+struct grub_efi_simple_network_mode
+{
+  grub_uint32_t state;
+  grub_uint32_t hwaddr_size;
+  grub_uint32_t media_header_size;
+  grub_uint32_t max_packet_size;
+  grub_uint32_t nvram_size;
+  grub_uint32_t nvram_access_size;
+  grub_uint32_t receive_filter_mask;
+  grub_uint32_t receive_filter_setting;
+  grub_uint32_t max_mcast_filter_count;
+  grub_uint32_t mcast_filter_count;
+  grub_efi_mac_t mcast_filter[16];
+  grub_efi_mac_t current_address;
+  grub_efi_mac_t broadcast_address;
+  grub_efi_mac_t permanent_address;
+  grub_uint8_t if_type;
+  grub_uint8_t mac_changeable;
+  grub_uint8_t multitx_supported;
+  grub_uint8_t media_present_supported;
+  grub_uint8_t media_present;
+};
+
+enum
+  {
+    GRUB_EFI_NETWORK_STOPPED,
+    GRUB_EFI_NETWORK_STARTED,
+    GRUB_EFI_NETWORK_INITIALIZED,
+  };
+
+struct grub_efi_simple_network
+{
+  grub_uint64_t revision;
+  grub_efi_status_t (*start) (struct grub_efi_simple_network *this);
+  void (*stop) (void);
+  grub_efi_status_t (*initialize) (struct grub_efi_simple_network *this,
+				   grub_efi_uintn_t extra_rx,
+				   grub_efi_uintn_t extra_tx);
+  void (*reset) (void);
+  void (*shutdown) (void);
+  void (*receive_filters) (void);
+  void (*station_address) (void);
+  void (*statistics) (void);
+  void (*mcastiptomac) (void);
+  void (*nvdata) (void);
+  grub_efi_status_t (*get_status) (struct grub_efi_simple_network *this,
+				   grub_uint32_t *int_status,
+				   void **txbuf);
+  grub_efi_status_t (*transmit) (struct grub_efi_simple_network *this,
+				 grub_efi_uintn_t header_size,
+				 grub_efi_uintn_t buffer_size,
+				 void *buffer,
+				 grub_efi_mac_t *src_addr,
+				 grub_efi_mac_t *dest_addr,
+				 grub_efi_uint16_t *protocol);
+  grub_efi_status_t (*receive) (struct grub_efi_simple_network *this,
+				grub_efi_uintn_t *header_size,
+				grub_efi_uintn_t *buffer_size,
+				void *buffer,
+				grub_efi_mac_t *src_addr,
+				grub_efi_mac_t *dest_addr,
+				grub_uint16_t *protocol);
+  void (*waitforpacket) (void);
+  struct grub_efi_simple_network_mode *mode;
+};
+typedef struct grub_efi_simple_network grub_efi_simple_network_t;
+
+
 struct grub_efi_block_io
 {
   grub_efi_uint64_t revision;
@@ -1128,7 +1392,7 @@ struct grub_efi_block_io
 };
 typedef struct grub_efi_block_io grub_efi_block_io_t;
 
-#if GRUB_TARGET_SIZEOF_VOID_P == 4
+#if (GRUB_TARGET_SIZEOF_VOID_P == 4) || defined (__ia64__)
 
 #define efi_call_0(func)		func()
 #define efi_call_1(func, a)		func(a)
@@ -1137,6 +1401,7 @@ typedef struct grub_efi_block_io grub_efi_block_io_t;
 #define efi_call_4(func, a, b, c, d)	func(a, b, c, d)
 #define efi_call_5(func, a, b, c, d, e)	func(a, b, c, d, e)
 #define efi_call_6(func, a, b, c, d, e, f) func(a, b, c, d, e, f)
+#define efi_call_7(func, a, b, c, d, e, f, g) func(a, b, c, d, e, f, g)
 #define efi_call_10(func, a, b, c, d, e, f, g, h, i, j)	func(a, b, c, d, e, f, g, h, i, j)
 
 #else
@@ -1144,24 +1409,31 @@ typedef struct grub_efi_block_io grub_efi_block_io_t;
 #define efi_call_0(func) \
   efi_wrap_0(func)
 #define efi_call_1(func, a) \
-  efi_wrap_1(func, (grub_uint64_t) a)
+  efi_wrap_1(func, (grub_uint64_t) (a))
 #define efi_call_2(func, a, b) \
-  efi_wrap_2(func, (grub_uint64_t) a, (grub_uint64_t) b)
+  efi_wrap_2(func, (grub_uint64_t) (a), (grub_uint64_t) (b))
 #define efi_call_3(func, a, b, c) \
-  efi_wrap_3(func, (grub_uint64_t) a, (grub_uint64_t) b, (grub_uint64_t) c)
+  efi_wrap_3(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
+	     (grub_uint64_t) (c))
 #define efi_call_4(func, a, b, c, d) \
-  efi_wrap_4(func, (grub_uint64_t) a, (grub_uint64_t) b, (grub_uint64_t) c, \
-  (grub_uint64_t) d)
+  efi_wrap_4(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
+	     (grub_uint64_t) (c), (grub_uint64_t) (d))
 #define efi_call_5(func, a, b, c, d, e)	\
-  efi_wrap_5(func, (grub_uint64_t) a, (grub_uint64_t) b, (grub_uint64_t) c, \
-  (grub_uint64_t) d, (grub_uint64_t) e)
+  efi_wrap_5(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
+	     (grub_uint64_t) (c), (grub_uint64_t) (d), (grub_uint64_t) (e))
 #define efi_call_6(func, a, b, c, d, e, f) \
-  efi_wrap_6(func, (grub_uint64_t) a, (grub_uint64_t) b, (grub_uint64_t) c, \
-  (grub_uint64_t) d, (grub_uint64_t) e, (grub_uint64_t) f)
+  efi_wrap_6(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
+	     (grub_uint64_t) (c), (grub_uint64_t) (d), (grub_uint64_t) (e), \
+	     (grub_uint64_t) (f))
+#define efi_call_7(func, a, b, c, d, e, f, g) \
+  efi_wrap_7(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
+	     (grub_uint64_t) (c), (grub_uint64_t) (d), (grub_uint64_t) (e), \
+	     (grub_uint64_t) (f), (grub_uint64_t) (g))
 #define efi_call_10(func, a, b, c, d, e, f, g, h, i, j) \
-  efi_wrap_10(func, (grub_uint64_t) a, (grub_uint64_t) b, (grub_uint64_t) c, \
-  (grub_uint64_t) d, (grub_uint64_t) e, (grub_uint64_t) f, (grub_uint64_t) g, \
-  (grub_uint64_t) h, (grub_uint64_t) i, (grub_uint64_t) j)
+  efi_wrap_10(func, (grub_uint64_t) (a), (grub_uint64_t) (b), \
+	      (grub_uint64_t) (c), (grub_uint64_t) (d), (grub_uint64_t) (e), \
+	      (grub_uint64_t) (f), (grub_uint64_t) (g),	(grub_uint64_t) (h), \
+	      (grub_uint64_t) (i), (grub_uint64_t) (j))
 
 grub_uint64_t EXPORT_FUNC(efi_wrap_0) (void *func);
 grub_uint64_t EXPORT_FUNC(efi_wrap_1) (void *func, grub_uint64_t arg1);
@@ -1179,6 +1451,10 @@ grub_uint64_t EXPORT_FUNC(efi_wrap_6) (void *func, grub_uint64_t arg1,
                                        grub_uint64_t arg2, grub_uint64_t arg3,
                                        grub_uint64_t arg4, grub_uint64_t arg5,
                                        grub_uint64_t arg6);
+grub_uint64_t EXPORT_FUNC(efi_wrap_7) (void *func, grub_uint64_t arg1,
+                                       grub_uint64_t arg2, grub_uint64_t arg3,
+                                       grub_uint64_t arg4, grub_uint64_t arg5,
+                                       grub_uint64_t arg6, grub_uint64_t arg7);
 grub_uint64_t EXPORT_FUNC(efi_wrap_10) (void *func, grub_uint64_t arg1,
                                         grub_uint64_t arg2, grub_uint64_t arg3,
                                         grub_uint64_t arg4, grub_uint64_t arg5,
