@@ -21,11 +21,21 @@
 
 #include <grub/err.h>
 #include <grub/types.h>
+#include <grub/list.h>
 
 /* Video color in hardware dependent format.  Users should not assume any
    specific coding format.  */
 typedef grub_uint32_t grub_video_color_t;
 
+/* Video color in hardware independent format.  */
+typedef struct grub_video_rgba_color
+{
+  grub_uint8_t red;
+  grub_uint8_t green;
+  grub_uint8_t blue;
+  grub_uint8_t alpha;
+} grub_video_rgba_color_t;
+
 /* This structure is driver specific and should not be accessed directly by
    outside code.  */
 struct grub_video_render_target;
@@ -37,20 +47,33 @@ struct grub_video_bitmap;
 /* If following is set render target contains currenly displayed image
    after swapping buffers (otherwise it contains previously displayed image).
  */
-#define GRUB_VIDEO_MODE_TYPE_UPDATING_SWAP	0x00000080
-#define GRUB_VIDEO_MODE_TYPE_PURE_TEXT		0x00000040
-#define GRUB_VIDEO_MODE_TYPE_ALPHA		0x00000020
-#define GRUB_VIDEO_MODE_TYPE_DOUBLE_BUFFERED	0x00000010
-#define GRUB_VIDEO_MODE_TYPE_1BIT_BITMAP	0x00000004
-#define GRUB_VIDEO_MODE_TYPE_INDEX_COLOR	0x00000002
-#define GRUB_VIDEO_MODE_TYPE_RGB		0x00000001
+typedef enum grub_video_mode_type
+  {
+    GRUB_VIDEO_MODE_TYPE_RGB              = 0x00000001,
+    GRUB_VIDEO_MODE_TYPE_INDEX_COLOR      = 0x00000002,
+    GRUB_VIDEO_MODE_TYPE_1BIT_BITMAP      = 0x00000004,
+    GRUB_VIDEO_MODE_TYPE_YUV              = 0x00000008,
 
-/* Defines used to mask flags.  */
-#define GRUB_VIDEO_MODE_TYPE_COLOR_MASK		0x0000000F
+    /* Defines used to mask flags.  */
+    GRUB_VIDEO_MODE_TYPE_COLOR_MASK       = 0x0000000F,
 
-/* Defines used to specify requested bit depth.  */
-#define GRUB_VIDEO_MODE_TYPE_DEPTH_MASK		0x0000ff00
-#define GRUB_VIDEO_MODE_TYPE_DEPTH_POS		8
+    GRUB_VIDEO_MODE_TYPE_DOUBLE_BUFFERED  = 0x00000010,
+    GRUB_VIDEO_MODE_TYPE_ALPHA            = 0x00000020,
+    GRUB_VIDEO_MODE_TYPE_PURE_TEXT        = 0x00000040,
+    GRUB_VIDEO_MODE_TYPE_UPDATING_SWAP    = 0x00000080,
+    GRUB_VIDEO_MODE_TYPE_OPERATIONAL_MASK = 0x000000F0,
+
+    /* Defines used to specify requested bit depth.  */
+    GRUB_VIDEO_MODE_TYPE_DEPTH_MASK       = 0x0000FF00,
+#define GRUB_VIDEO_MODE_TYPE_DEPTH_POS	  8
+
+    GRUB_VIDEO_MODE_TYPE_UNKNOWN          = 0x00010000,
+    GRUB_VIDEO_MODE_TYPE_HERCULES         = 0x00020000,
+    GRUB_VIDEO_MODE_TYPE_PLANAR           = 0x00040000,
+    GRUB_VIDEO_MODE_TYPE_NONCHAIN4        = 0x00080000,
+    GRUB_VIDEO_MODE_TYPE_CGA              = 0x00100000,
+    GRUB_VIDEO_MODE_TYPE_INFO_MASK        = 0x00FF0000,
+  } grub_video_mode_type_t;
 
 /* The basic render target representing the whole display.  This always
    renders to the back buffer when double-buffering is in use.  */
@@ -102,7 +125,7 @@ struct grub_video_mode_info
 
   /* Mode type bitmask.  Contains information like is it Index color or
      RGB mode.  */
-  unsigned int mode_type;
+  grub_video_mode_type_t mode_type;
 
   /* Bits per pixel.  */
   unsigned int bpp;
@@ -116,6 +139,9 @@ struct grub_video_mode_info
   /* In index color mode, number of colors.  In RGB mode this is 256.  */
   unsigned int number_of_colors;
 
+  unsigned int mode_number;
+#define GRUB_VIDEO_MODE_NUMBER_INVALID 0xffffffff
+
   /* Optimization hint how binary data is coded.  */
   enum grub_video_blit_format blit_format;
 
@@ -167,6 +193,15 @@ struct grub_video_rect
 };
 typedef struct grub_video_rect grub_video_rect_t;
 
+struct grub_video_signed_rect
+{
+  signed x;
+  signed y;
+  unsigned width;
+  unsigned height;
+};
+typedef struct grub_video_signed_rect grub_video_signed_rect_t;
+
 struct grub_video_palette_data
 {
   grub_uint8_t r; /* Red color value (0-255).  */
@@ -175,20 +210,102 @@ struct grub_video_palette_data
   grub_uint8_t a; /* Reserved bits value (0-255).  */
 };
 
+struct grub_video_edid_info
+{
+  grub_uint8_t header[8];
+  grub_uint16_t manufacturer_id;
+  grub_uint16_t product_id;
+  grub_uint32_t serial_number;
+  grub_uint8_t week_of_manufacture;
+  grub_uint8_t year_of_manufacture;
+  grub_uint8_t version;
+  grub_uint8_t revision;
+
+  grub_uint8_t video_input_definition;
+  grub_uint8_t max_horizontal_image_size;
+  grub_uint8_t max_vertical_image_size;
+  grub_uint8_t display_gamma;
+  grub_uint8_t feature_support;
+#define GRUB_VIDEO_EDID_FEATURE_PREFERRED_TIMING_MODE	(1 << 1)
+
+  grub_uint8_t red_green_lo;
+  grub_uint8_t blue_white_lo;
+  grub_uint8_t red_x_hi;
+  grub_uint8_t red_y_hi;
+  grub_uint8_t green_x_hi;
+  grub_uint8_t green_y_hi;
+  grub_uint8_t blue_x_hi;
+  grub_uint8_t blue_y_hi;
+  grub_uint8_t white_x_hi;
+  grub_uint8_t white_y_hi;
+
+  grub_uint8_t established_timings_1;
+  grub_uint8_t established_timings_2;
+  grub_uint8_t manufacturer_reserved_timings;
+
+  grub_uint16_t standard_timings[8];
+
+  struct {
+    grub_uint16_t pixel_clock;
+    /* Only valid if the pixel clock is non-null.  */
+    grub_uint8_t horizontal_active_lo;
+    grub_uint8_t horizontal_blanking_lo;
+    grub_uint8_t horizontal_hi;
+    grub_uint8_t vertical_active_lo;
+    grub_uint8_t vertical_blanking_lo;
+    grub_uint8_t vertical_hi;
+    grub_uint8_t horizontal_sync_offset_lo;
+    grub_uint8_t horizontal_sync_pulse_width_lo;
+    grub_uint8_t vertical_sync_lo;
+    grub_uint8_t sync_hi;
+    grub_uint8_t horizontal_image_size_lo;
+    grub_uint8_t vertical_image_size_lo;
+    grub_uint8_t image_size_hi;
+    grub_uint8_t horizontal_border;
+    grub_uint8_t vertical_border;
+    grub_uint8_t flags;
+  } detailed_timings[4];
+
+  grub_uint8_t extension_flag;
+  grub_uint8_t checksum;
+} __attribute__ ((packed));
+
 typedef enum grub_video_driver_id
   {
     GRUB_VIDEO_DRIVER_NONE,
     GRUB_VIDEO_DRIVER_VBE,
     GRUB_VIDEO_DRIVER_EFI_UGA,
-    GRUB_VIDEO_DRIVER_EFI_GOP
+    GRUB_VIDEO_DRIVER_EFI_GOP,
+    GRUB_VIDEO_DRIVER_SM712,
+    GRUB_VIDEO_DRIVER_VGA,
+    GRUB_VIDEO_DRIVER_CIRRUS,
+    GRUB_VIDEO_DRIVER_BOCHS,
+    GRUB_VIDEO_DRIVER_SDL,
+    GRUB_VIDEO_DRIVER_SIS315PRO,
+    GRUB_VIDEO_DRIVER_RADEON_FULOONG2E,
   } grub_video_driver_id_t;
 
+typedef enum grub_video_adapter_prio
+  {
+    GRUB_VIDEO_ADAPTER_PRIO_FALLBACK = 60,
+    GRUB_VIDEO_ADAPTER_PRIO_FIRMWARE_DIRTY = 70,
+    GRUB_VIDEO_ADAPTER_PRIO_FIRMWARE = 80,
+    GRUB_VIDEO_ADAPTER_PRIO_NATIVE = 100
+  } grub_video_adapter_prio_t;
+
+
 struct grub_video_adapter
 {
+  /* The next video adapter.  */
+  struct grub_video_adapter *next;
+  struct grub_video_adapter **prev;
+
   /* The video adapter name.  */
   const char *name;
   grub_video_driver_id_t id;
 
+  grub_video_adapter_prio_t prio;
+
   /* Initialize the video adapter.  */
   grub_err_t (*init) (void);
 
@@ -196,7 +313,8 @@ struct grub_video_adapter
   grub_err_t (*fini) (void);
 
   grub_err_t (*setup) (unsigned int width,  unsigned int height,
-                       unsigned int mode_type, unsigned int mode_mask);
+                       grub_video_mode_type_t mode_type,
+		       grub_video_mode_type_t mode_mask);
 
   grub_err_t (*get_info) (struct grub_video_mode_info *mode_info);
 
@@ -254,14 +372,37 @@ struct grub_video_adapter
 
   grub_err_t (*get_active_render_target) (struct grub_video_render_target **target);
 
-  /* The next video adapter.  */
-  struct grub_video_adapter *next;
+  int (*iterate) (int (*hook) (const struct grub_video_mode_info *info));
+
+  grub_err_t (*get_edid) (struct grub_video_edid_info *edid_info);
+
+  void (*print_adapter_specific_info) (void);
 };
 typedef struct grub_video_adapter *grub_video_adapter_t;
 
-void EXPORT_FUNC (grub_video_register) (grub_video_adapter_t adapter);
-void grub_video_unregister (grub_video_adapter_t adapter);
-void grub_video_iterate (int (*hook) (grub_video_adapter_t adapter));
+extern grub_video_adapter_t EXPORT_VAR(grub_video_adapter_list);
+
+#ifndef GRUB_LST_GENERATOR
+/* Register video driver.  */
+static inline void
+grub_video_register (grub_video_adapter_t adapter)
+{
+  grub_video_adapter_t *p;
+  for (p = &grub_video_adapter_list; *p && (*p)->prio > adapter->prio; 
+       p = &((*p)->next));
+  adapter->next = *p;
+  *p = adapter;
+}
+#endif
+
+/* Unregister video driver.  */
+static inline void
+grub_video_unregister (grub_video_adapter_t adapter)
+{
+  grub_list_remove (GRUB_AS_LIST (adapter));
+}
+
+#define FOR_VIDEO_ADAPTERS(var) FOR_LIST_ELEMENTS((var), (grub_video_adapter_list))
 
 grub_err_t EXPORT_FUNC (grub_video_restore) (void);
 
@@ -272,7 +413,7 @@ grub_err_t EXPORT_FUNC (grub_video_get_info) (struct grub_video_mode_info *mode_
    sure that framebuffer address doesn't change. To ensure this abstraction
    grub_video_get_info_and_fini is the only function supplying framebuffer
    address. */
-grub_err_t grub_video_get_info_and_fini (struct grub_video_mode_info *mode_info,
+grub_err_t EXPORT_FUNC (grub_video_get_info_and_fini) (struct grub_video_mode_info *mode_info,
 					 void **framebuffer);
 
 enum grub_video_blit_format grub_video_get_blit_format (struct grub_video_mode_info *mode_info);
@@ -280,8 +421,9 @@ enum grub_video_blit_format grub_video_get_blit_format (struct grub_video_mode_i
 grub_err_t grub_video_set_palette (unsigned int start, unsigned int count,
                                    struct grub_video_palette_data *palette_data);
 
-grub_err_t grub_video_get_palette (unsigned int start, unsigned int count,
-                                   struct grub_video_palette_data *palette_data);
+grub_err_t EXPORT_FUNC (grub_video_get_palette) (unsigned int start,
+						 unsigned int count,
+						 struct grub_video_palette_data *palette_data);
 
 grub_err_t EXPORT_FUNC (grub_video_set_viewport) (unsigned int x,
 						  unsigned int y,
@@ -345,18 +487,59 @@ grub_err_t EXPORT_FUNC (grub_video_set_active_render_target) (struct grub_video_
 
 grub_err_t grub_video_get_active_render_target (struct grub_video_render_target **target);
 
+grub_err_t EXPORT_FUNC (grub_video_edid_checksum) (struct grub_video_edid_info *edid_info);
+grub_err_t EXPORT_FUNC (grub_video_edid_preferred_mode) (struct grub_video_edid_info *edid_info,
+					   unsigned int *width,
+					   unsigned int *height);
+
 grub_err_t EXPORT_FUNC (grub_video_set_mode) (const char *modestring,
 					      unsigned int modemask,
 					      unsigned int modevalue);
 
 static inline int
-grub_video_check_mode_flag (unsigned int flags, unsigned int mask,
-			    unsigned int flag, int def)
+grub_video_check_mode_flag (grub_video_mode_type_t flags,
+			    grub_video_mode_type_t mask,
+			    grub_video_mode_type_t flag, int def)
 {
   return (flag & mask) ? !! (flags & flag) : def;
 }
 
-grub_video_driver_id_t
-grub_video_get_driver_id (void);
+grub_video_driver_id_t EXPORT_FUNC (grub_video_get_driver_id) (void);
+
+static __inline grub_video_rgba_color_t
+grub_video_rgba_color_rgb (int r, int g, int b)
+{
+  grub_video_rgba_color_t c;
+  c.red = r;
+  c.green = g;
+  c.blue = b;
+  c.alpha = 255;
+  return c;
+}
+
+static __inline grub_video_color_t
+grub_video_map_rgba_color (grub_video_rgba_color_t c)
+{
+  return grub_video_map_rgba (c.red, c.green, c.blue, c.alpha);
+}
+
+int EXPORT_FUNC (grub_video_get_named_color) (const char *name,
+					      grub_video_rgba_color_t *color);
+
+grub_err_t EXPORT_FUNC (grub_video_parse_color) (const char *s,
+						 grub_video_rgba_color_t *color);
+
+#ifndef GRUB_MACHINE_EMU
+extern void grub_font_init (void);
+extern void grub_font_fini (void);
+extern void grub_gfxterm_init (void);
+extern void grub_gfxterm_fini (void);
+extern void grub_video_sm712_init (void);
+extern void grub_video_sm712_fini (void);
+extern void grub_video_sis315pro_init (void);
+extern void grub_video_radeon_fuloong2e_init (void);
+extern void grub_video_sis315pro_fini (void);
+extern void grub_video_radeon_fuloong2e_fini (void);
+#endif
 
 #endif /* ! GRUB_VIDEO_HEADER */
